classdef FolderNHF<handle

  properties
      folderFullPath
      fldNHFData
      fldNHFHeader
  end

  methods
    function obj = FolderNHF(fullPath)
      obj.folderFullPath = fullPath;
			obj.parserFolderInfo();
    end

		function parserFolderInfo(obj)
			fileNFIList = dir(fullfile(obj.folderFullPath, '*.nfi'));
      if length(fileNFIList) < 1
        disp('No *.nfi file was found.');
        return;
      end
      nfiName = fileNFIList(1).name;
      folderCat = fileNFI.GetFolderCategory(fullfile(obj.folderFullPath, nfiName));
      header.folderCat = folderCat;
      fileNHFList = dir(fullfile(obj.folderFullPath, '*.nhf'));
      if length(fileNHFList) < 1
        disp('No *.nhf file was found.');
        return;
      end
      nhfName = fileNHFList(1).name;
      nhf = fileNHF(fullfile(obj.folderFullPath, nhfName));
      nhfInfo = nhf.nhfInfo;
      header.nhfInfo = nhfInfo;
      hCount = nhfInfo.hostCount;
      fldNSFObjAr = [];
      chIdx2fldNsfObjAr = [];
      durationsAr = zeros(1,hCount);
      chNamesNHFLevelAr = string([]);%NHF level channel names
      chCountsNHFLevel = 0;
      sampleRateAr = [];
      chNamesNSFLevelAr = string([]);
      chNameMapDic = containers.Map();
      chflNSFObjIndexMapDic = containers.Map();
      chsInfo = [];
      rSegs = [];
      rSegCnt = 0;
      for i = 1:hCount
        h = nhfInfo.hosts(i);
        %hostFolder = strcat(h.folder');%h.name
        hostFolder = h.folder;%h.name
        fldNSF = FolderNSF(fullfile(obj.folderFullPath,hostFolder));
        fldNSFObjAr = [fldNSFObjAr fldNSF];
        fldNSFHeader = fldNSF.folderNSFInfo;
        chsInfo = [chsInfo fldNSFHeader.chsInfo];
        durationsAr(i) = fldNSFHeader.duration;
        names = fldNSFHeader.channelNames;
        chNamesNSFLevelAr = [chNamesNSFLevelAr names];
        chRep = ismember(names,chNamesNHFLevelAr);
        if nhfInfo.IsHostNameASCII
          names(chRep) = strcat(hostFolder,"_",names(chRep));
        else
          hostNameASCII = nhfInfo.hostNameCodeMap(hostFolder);
          names(chRep) = strcat(hostNameASCII,"_",names(chRep));
        end
        %names(chRep) = strcat(hostFolder,"_",names(chRep));
        chNamesNHFLevelAr = [chNamesNHFLevelAr names];
        chCount = fldNSFHeader.channelCount;
        objIndex = i*ones(1,chCount);
        chIdx2fldNsfObjAr = [chIdx2fldNsfObjAr objIndex];
        for j = 1:chCount
          chNameMapDic(char(names(j))) = fldNSFHeader.channelNames(j);
          chflNSFObjIndexMapDic(char(names(j))) = objIndex(j);
        end
        chCountsNHFLevel = chCountsNHFLevel + chCount;
        sampleRateAr = [sampleRateAr fldNSFHeader.sampleRate];
        segCnt = fldNSFHeader.segCount;
        if segCnt>1
          rSegs = [rSegs fldNSFHeader.recordSegs];
          rSegCnt = rSegCnt+segCnt;
        end
      end

      header.patientName = fldNSFObjAr(1).folderNSFInfo.patientName;
      header.subID = strcat(char( fldNSFObjAr(1).folderNSFInfo.subID)');
      header.hostCount = hCount;
      header.recordSegs = rSegs;
      header.segCount = rSegCnt;
      header.fldNSFObjAr = fldNSFObjAr;
      header.chNamesNHFLevelAr = chNamesNHFLevelAr;
      header.chNamesNSFLevelAr = chNamesNSFLevelAr;
      header.chIdx2fldNsfObjAr = chIdx2fldNsfObjAr;
      header.chNameMapDic = chNameMapDic;
      header.sampleRateAr  = sampleRateAr;
      header.chCountsNHFLevel = chCountsNHFLevel;
      header.chs = chsInfo;
      header.durationsAr = durationsAr;
      header.duration = max(durationsAr);
      header.chflNSFObjIndexMapDic = chflNSFObjIndexMapDic;
      obj.fldNHFHeader = header;
		end

    function fdInfo = getFolderInfo(obj,varargin)
      % 20230718 增加单独选择一个参数情况
      % varargin:varargin{1} 参数类型；varargin{2}，对应通道位置；varargin{3}，采样率
      % else部分为新加内容
			nVarargs = length(varargin);
      hd = obj.fldNHFHeader;
      fdInfo = FolderInformation(hd.folderCat);
      fdInfo.patientID = hd.subID;
      fdInfo.patientName = hd.patientName;

      if nVarargs==0
        fdInfo.sRate = max(hd.sampleRateAr);
        fdInfo.duration = hd.duration;
        fdInfo.channels = hd.chs;
        fdInfo.channelCount = hd.chCountsNHFLevel;
        fdInfo.channelNames = hd.chNamesNHFLevelAr;
        fdInfo.recordSegs = hd.recordSegs;
        fdInfo.segCount = hd.segCount;
      elseif strcmpi(varargin{1},'All')
        fdInfo.sRate = max(hd.sampleRateAr);
        fdInfo.duration = hd.duration;
        fdInfo.channels = hd.chs;
        fdInfo.channelCount = hd.chCountsNHFLevel;
        fdInfo.channelNames = hd.chNamesNHFLevelAr;
        fdInfo.recordSegs = hd.recordSegs;
        fdInfo.segCount = hd.segCount;
      else
        Chsel =  varargin{2};
        fdInfo.sRate = varargin{3};
        fdInfo.duration = hd.duration;
        fdInfo.channels = hd.chs(Chsel);
        fdInfo.channelCount = length(Chsel);
        fdInfo.channelNames = hd.chNamesNHFLevelAr(Chsel);
        fdInfo.recordSegs = hd.recordSegs;
        fdInfo.segCount = hd.segCount;

        obj.fldNHFHeader.sampleRateAr = obj.fldNHFHeader.sampleRateAr(Chsel);
        obj.fldNHFHeader.chCountsNHFLevel = fdInfo.channelCount;
        obj.fldNHFHeader.chNamesNHFLevelAr = fdInfo.channelNames;
        obj.fldNHFHeader.chNamesNSFLevelAr = fdInfo.channelNames;
        obj.fldNHFHeader.chs = fdInfo.channels;
        obj.fldNHFHeader.chIdx2fldNsfObjAr = obj.fldNHFHeader.chIdx2fldNsfObjAr(Chsel);

        chNameMapDic = containers.Map();
        chflNSFObjIndexMapDic = containers.Map();
        for j = 1:fdInfo.channelCount
          chNameMapDic(char(fdInfo.channelNames(j))) = fdInfo.channelNames(j);
          chflNSFObjIndexMapDic(char(fdInfo.channelNames(j))) = obj.fldNHFHeader.chIdx2fldNsfObjAr(j);
        end
        obj.fldNHFHeader.chNameMapDic = chNameMapDic;
        obj.fldNHFHeader.chflNSFObjIndexMapDic = chflNSFObjIndexMapDic;
      end
    end
		
    function data = getFolderData(obj,varargin)
			hd = obj.fldNHFHeader;
      %prepare the data struct in NHF folder Level
      chs = hd.chs;
      chCountAll = hd.chCountsNHFLevel;
      durAll = max(hd.durationsAr);
			channelNames = hd.chNamesNHFLevelAr;

			startReadSec = 0;
			durationRead = durAll;
			chCntRead = chCountAll;
			chNamesRead = string(chCountAll);
			chNameInfoDic = containers.Map();
			for i=1:chCountAll
        chN = channelNames(i);
				chNamesRead(i) = chN;
				chNameInfoDic(char(chN)) = chs(i);
      end

			isAllChannel = true;
			isAllTimeRange = true;
			nVarargs = length(varargin);
			if(nVarargs == 1)
				readOption = varargin{1};
				isAllChannel = readOption.isAllChannels;
				isAllTimeRange = readOption.isAllTimeRange;
			end

			if isAllChannel && isAllTimeRange
				data = obj.getFolderAllData();
      elseif isAllChannel
				startReadSec = readOption.startSec;
				durationRead = readOption.duration;
				data = obj.getFolderDataTimeRange(startReadSec,durationRead);
			else
				startReadSec = readOption.startSec;
				durationRead = readOption.duration;
				chNamesRead = readOption.channelNames;
				data = obj.getFolderDataTimeRangePartialChannal(startReadSec,durationRead,chNamesRead);
			end
			obj.fldNHFData = data;
    end

	function dataAll = getFolderAllData(obj)
		hd = obj.fldNHFHeader;
      %prepare the data struct in NHF folder Level
      chs = hd.chs;
      chCountAll = hd.chCountsNHFLevel;
      durAll = max(hd.durationsAr);
			channelNames = hd.chNamesNHFLevelAr;
			channelNamesNSFLevel = hd.chNamesNHFLevelAr;
      durations = hd.durationsAr;
			sampleRate = hd.sampleRateAr;
      durationNHF = max(durations);
      sampleRateNHF = sampleRate(1);
      sr = unique(sampleRate)
			isResample = false;
      if length(sr) ~= 1
          sampleRateNHF = max(sampleRate);
          isResample = true;
      end

      fldNSFObjAr = hd.fldNSFObjAr;
			chNamesNSFLevelAr = hd.chNamesNSFLevelAr;
			chIdx2fldNsfObjAr = hd.chIdx2fldNsfObjAr;
      dataAll = zeros(chCountAll,durationNHF*sampleRateNHF);
      for i = 1:chCountAll
		nmNsf = chNamesNSFLevelAr(i);
        % 20230725 debug 数据中存在不同探头下的同名通道时，低速探头通道名称存在后缀问题
        % 203231221 debug 下划线区分通道名称导致异常，修改成只针对ECG通道
        if contains(nmNsf,'ECG')
            apart_nmNsf = split(nmNsf,'_');
            nmNsf = apart_nmNsf(end);
        end

        nsfFldObj = fldNSFObjAr(chIdx2fldNsfObjAr(i));
        if isempty(nsfFldObj.folderNSFDataDic)
            nsfFldObj.getFolderAllData();
        end
        dataNSFFld = nsfFldObj.folderNSFDataDic;
		srChI = chs(i).SampleRate;
        dataNSF = dataNSFFld(char(nmNsf));
		dataNSFVector = reshape(dataNSF,1,[]);
		if(srChI~=sampleRateNHF)
            stpNHF = 1/sampleRateNHF;
			x1 = stpNHF:stpNHF:durationNHF;
			stpNSF = 1/srChI;
			x = stpNSF:stpNSF:durationNHF;
			dataNSFVector = interp1(x,dataNSFVector,x1);
            dataNSFVector(isnan(dataNSFVector))=0;
		end
        dataAll(i,:) = dataNSFVector;
      end
		end

		function dataTimeRange = getFolderDataTimeRange(obj,startReadSec,durationRead)
			hd = obj.fldNHFHeader;
      %prepare the data struct in NHF folder Level
      chs = hd.chs;
      chCountAll = hd.chCountsNHFLevel;
      durAll = max(hd.durationsAr);
			channelNames = hd.chNamesNHFLevelAr;
			channelNamesNSFLevel = hd.chNamesNHFLevelAr;
			sampleRate = hd.sampleRateAr;
      durationNHF = max(durAll);
      sampleRateNHF = sampleRate(1);
      sr = unique(sampleRate);
			isResample = false;
      if length(sr) ~= 1
        sampleRateNHF = max(sampleRate);
        isResample = true;
      end
      fldNSFObjAr = hd.fldNSFObjAr;
			chNamesNSFLevelAr = hd.chNamesNSFLevelAr;
			chIdx2fldNsfObjAr = hd.chIdx2fldNsfObjAr;
			durationNHF = durationRead;
      dataTimeRange = zeros(chCountAll,durationNHF*sampleRateNHF);
      for i = 1:chCountAll
		nmNsf = char(chNamesNSFLevelAr(i));
        % 20230725 debug 数据中存在不同探头下的同名通道时，低速探头通道名称存在后缀问题
        apart_nmNsf = split(nmNsf,'_');
        nmNsf = apart_nmNsf{end};

        nsfFldObj = fldNSFObjAr(chIdx2fldNsfObjAr(i));
        if isempty(nsfFldObj.folderNSFDataDic)
          nsfFldObj.getFolderDataTimeRange(startReadSec,durationRead);
        end
        dataNSFFld = nsfFldObj.folderNSFDataDic;
				srChI = chs(i).SampleRate;
        dataNSF = dataNSFFld(nmNsf);
				dataNSFVector = reshape(dataNSF,1,[]);
				if(srChI~=sampleRateNHF)
					stpNHF = 1/sampleRateNHF;
					x1 = stpNHF:stpNHF:durationNHF;
					stpNSF = 1/srChI;
					x = stpNSF:stpNSF:durationNHF;
					dataNSFVector = interp1(x,dataNSFVector,x1);
          dataNSFVector(isnan(dataNSFVector))=0;
				end
        dataTimeRange(i,:) = dataNSFVector;
      end
		end

		function dataTimeRange = getFolderDataTimeRangePartialChannal(obj,startReadSec,durationRead,chNamesRead)
			hd = obj.fldNHFHeader;
      %prepare the data struct in NHF folder Level
      chs = hd.chs;
      chCountAll = hd.chCountsNHFLevel;
      durAll = max(hd.durationsAr);
			channelNames = hd.chNamesNHFLevelAr;
      index = find(ismember(channelNames,chNamesRead))
			channelNamesNSFLevel = hd.chNamesNHFLevelAr;
			sampleRate = hd.sampleRateAr;
      durationNHF = max(durAll);
      sampleRateNHF = sampleRate(index);
      sr = unique(sampleRateNHF);
			isResample = false;
      sampleRateNHF = max(sampleRate);
      if length(sr) ~= 1
        isResample = true;
      end

			chCountRead = length(chNamesRead);
 			chNameMapDic = hd.chNameMapDic;           
			flNSFObjDic = hd.chflNSFObjIndexMapDic;
      fldNSFObjAr = hd.fldNSFObjAr;
			chNamesNSFLevelAr = hd.chNamesNSFLevelAr;
			chIdx2fldNsfObjAr = hd.chIdx2fldNsfObjAr;
      if(durationRead>1)
        durationNHF = durationRead;
      end
      dataTimeRange = zeros(chCountRead,durationNHF*sampleRateNHF);
			names = string(chCountRead);
      nameGroup = zeros(chCountRead);
      for i=1:chCountRead
        if(ischar(chNamesRead(i)))
          disp("Channel name format is not correct.");
          return;
        else
          nameI = char(strcat(chNamesRead(i)));
        end
        names(i) = chNameMapDic(nameI);
        nameGroup(i)= flNSFObjDic(nameI);
      end

      for i = 1:chCountRead
        if(ischar(chNamesRead(i)))
          nmNsf = chNamesRead;
        else
          nmNsf = char(strcat(chNamesRead(i)));
        end

        nsfFldObjIndex = flNSFObjDic(nmNsf);
        nsfFldObj = fldNSFObjAr(nsfFldObjIndex);

        if isempty(nsfFldObj.folderNSFDataDic)
          chsRead = names(nameGroup==nsfFldObjIndex);
          % 20230725 debug 数据中存在不同探头下的同名通道时，低速探头通道名称存在后缀问题
          for ich = 1:length(chsRead)
            apart_chsRead = split(chsRead(ich),'_'); % 20230829 debug
            chsRead(ich) = apart_chsRead(end); % 20230829 debug
          end
          nsfFldObj.getFolderDataTimeRangePartialChannal(startReadSec,durationRead,chsRead);
        end

        nm = char(names(i));
        nm_apart = split(nm,'_');% 20230824
        nm = nm_apart{end};
        dataNSFFld = nsfFldObj.folderNSFDataDic;
				srChI = nsfFldObj.folderNSFInfo.chDic(nm).SampleRate;
        dataNSF = dataNSFFld(nm);
				dataNSFVector = reshape(dataNSF,1,[]);
				if(srChI~=sampleRateNHF)
          stpNHF = 1/sampleRateNHF;
					x1 = stpNHF:stpNHF:durationNHF;
					stpNSF = 1/srChI;
					x = stpNSF:stpNSF:durationNHF;
					dataNSFVector = interp1(x,dataNSFVector,x1);
          dataNSFVector(isnan(dataNSFVector))=0;
				end
        dataTimeRange(i,:) = dataNSFVector;
      end
		end

  end
end

