classdef FolderNSF<handle
    properties
        folderPath
        folderNSFInfo
		folderNSFDataDic
		folderNSFDataAr
		folderNSFDataDicChNames
    end
    
    methods
        function obj = FolderNSF(fullPath)
            obj.folderPath = fullPath;
			obj.parseFolderInfo();
        end
        
		function parseFolderInfo(obj)
            fileNFIList = dir(fullfile(obj.folderPath, '*.nfi'));
            if length(fileNFIList) < 1
                disp('No *.nfi file was found.');
                return;
            end
            nfiName = fileNFIList(1).name;
            header.folderCat = fileNFI.GetFolderCategory(fullfile(obj.folderPath, nfiName));
            fileNSFList = dir(fullfile(obj.folderPath, '*.nsf'));
            if length(fileNSFList) < 1
                disp('No *.nhf file was found.');
                return;
            end
            nsfName = fileNSFList(1).name;
            nsf = fileNSF(fullfile(obj.folderPath, nsfName));
			nsfInfo = nsf.nsfInfo; 
            header.nsfInfo = nsfInfo;
            segCnt = nsfInfo.segCount;
            header.segCount = segCnt;
            header.recordSegs = nsfInfo.segs;
            %prpare the data struct in NSF folder Level
            chNameDic = containers.Map();
            chs = nsfInfo.channels;  
            chCount = nsfInfo.chCount;
            dur = nsfInfo.DurationCal;%DurationCal
            header.channelCount = chCount;
            header.channels = chs;
            header.duration = dur;
            names = string(chCount);
            chSR = zeros(1,chCount);
            for i=1:chCount
                chname = chs(i).ChannelName;
                names(i) = chname;
                chSR(i) = chs(i).SampleRate;
				chNameDic(chname) = chs(i);
            end
			header.patientName = nsfInfo.name;
			header.subID = nsfInfo.subID;
            header.channelNames= names;
            header.sampleRate = chSR;
			header.chDic = chNameDic;
			header.chsInfo = chs;
            obj.folderNSFInfo = header;
        end      

        function fdInfo = getFolderInfo(obj)
			hd = obj.folderNSFInfo;
            fdInfo = FolderInformation(hd.folderCat);
			fdInfo.patientID = hd.subID;
			fdInfo.patientName = hd.patientName;
			fdInfo.sRate = max(hd.sampleRate);
			fdInfo.duration = hd.duration;
			fdInfo.channels = hd.channels;
			fdInfo.channelCount = hd.channelCount;
			fdInfo.channelNames = hd.channelNames;
            fdInfo.recordSegs = hd.recordSegs;
            fdInfo.segCount = hd.segCount;
			%obj.folderNSFInfo = fdInfo;
        end

        function dataMatrix = getFolderData(obj,varargin)
			hd = obj.folderNSFInfo;
            nsfInfo = hd.nsfInfo;
            
            %prepare the data struct in NSF folder Level  
            chs = nsfInfo.channels;  
            chCountAll = nsfInfo.chCount;
            durAll = nsfInfo.DurationCal;
			
			chNameInfoDic = nsfInfo.chInfoDic;
			startReadSec = 0;
			durationRead = durAll;
			chCntRead = chCountAll;
			chNamesRead = string(chCountAll);
			chSRAr = zeros(1,chCntRead);
			for i=1:chCountAll
                chN = chs(i).ChannelName;
				chNamesRead(i) = chN;
				chSRAr(i) = chs(i).SampleRate;
            end

			sampleRateNSF = max(chSRAr);
			isAllChannel = true;
			isAllTimeRange = true;
			nVarargs = length(varargin);
			if(nVarargs == 1)
				readOption = varargin{1};
				isAllChannel = readOption.isAllChannels;
				isAllTimeRange = readOption.isAllTimeRange;
			end

			if isAllChannel && isAllTimeRange
				data = obj.getFolderAllData();
            elseif isAllChannel
				startReadSec = readOption.startSec;
				durationRead = readOption.duration;
				data = obj.getFolderDataTimeRange(startReadSec,durationRead);

			else
				startReadSec = readOption.startSec;
				durationRead = readOption.duration;
				chNamesRead = readOption.channelNames;
				data = obj.getFolderDataTimeRangePartialChannal(startReadSec,durationRead,chNamesRead);
			end
			obj.folderNSFDataDic = data;
			obj.folderNSFDataDicChNames = chNamesRead;
			chReadCnt = length(chNamesRead);
			dataAr = zeros(chReadCnt,sampleRateNSF*durationRead);
			for i=1:chReadCnt
                chN = char(chNamesRead(i));
				chInfo = chNameInfoDic(chN);
				srChI = chInfo.SampleRate;
				dt = data(chN);
				dataVector = reshape(dt,1,[]);
				if(srChI~=sampleRateNSF)
					stp1 = 1/sampleRateNSF;
					x1 = stp1:stp1:durationRead;
					stp = 1/srChI;
					x = stp:stp:durationRead;
					dataVector = interp1(x,dataVector,x1);
                    dataVector(isnan(dataVector))=0;
				end
				dataAr(i,:) = dataVector;
            end
            dataMatrix = dataAr;
			obj.folderNSFDataAr = dataAr;
        end
        
        function data = getFolderAllData(obj)
			hd = obj.folderNSFInfo;
            nsfInfo = hd.nsfInfo;
            
            %prepare the data struct in NSF folder Level  
            chs = nsfInfo.channels;  
            chCountAll = nsfInfo.chCount;
            durAll = nsfInfo.DurationCal;%DurationCal
			startReadSec = 0;
			durationRead = durAll;
			chCntRead = chCountAll;
			chNamesRead = string(chCountAll);
			
			chNameDataDic = containers.Map();
            for i=1:chCntRead
                chInfo = chs(i);
                rows = chInfo.SampleRate;
                cols = durationRead;
                chNameDataDic(chInfo.ChannelName) = zeros(rows,cols);
            end
            %merge
            segCnt = nsfInfo.segCount;
            for i = 1:segCnt
                seg = nsfInfo.segs(i);
                fName = seg.SegmentFolderName;
                start = seg.StartSecond+1;
                dur = seg.Duration;
                stop =  seg.StartSecond+dur;
                ndf = fileNDF(fullfile(obj.folderPath, fName));
                dataSeg = ndf.readAllPhysicalValues();
                for j = 1:chCountAll
                    chname = chs(j).ChannelName;
                    TF = isKey(dataSeg,chname);
                    if TF
                        tmp = chNameDataDic(chname);
                        tmp(:,start:stop) = dataSeg(chname);
                        chNameDataDic(chname) = tmp;
                    end
                end
            end
            obj.folderNSFDataDic = chNameDataDic;
            data = chNameDataDic;
        end

        function data = getFolderDataTimeRange(obj,startSec,durationSec)
			hd = obj.folderNSFInfo;
            nsfInfo = hd.nsfInfo;
            
            %prepare the data struct in NSF folder Level  
            chs = nsfInfo.channels;  
            chCountAll = nsfInfo.chCount;
            durAll = nsfInfo.DurationCal;
          
			startReadSec = startSec;
			durationRead = durationSec;
			if(durationSec>durAll)
				durationRead = durAll;
			end
			chCntRead = chCountAll;
			chNamesRead = string(chCountAll);
			stopReadSec = startReadSec + durationRead;
			chNameDataDic = containers.Map();
            for i=1:chCntRead
                chInfo = chs(i);
                rows = chInfo.SampleRate;
                cols = durationRead;
                chNameDataDic(chInfo.ChannelName) = zeros(rows,cols);
            end
            %merge
            segCnt = obj.folderNSFInfo.segCount;
            for i = 1:segCnt
                seg = nsfInfo.segs(i);
                fName = seg.SegmentFolderName;
                start = seg.StartSecond;
                dur = seg.Duration;
				stop = start + dur;
				if start >= stopReadSec
					continue;
				end

				if stop <=startReadSec
					continue;
				end

				startNew = start;
				if start < startReadSec
					startNew = startReadSec;
				end

				stopNew = stop;
				if stop > stopReadSec
					stopNew = stopReadSec;
				end
				durNew = stopNew - startNew;
                fReadOff = startNew - start;
                ndf = fileNDF(fullfile(obj.folderPath, fName));
                dataSeg = ndf.readPeriodPhysicalValues(fReadOff,durNew);
                pOffset = startNew - startSec;
                for j = 1:chCntRead
                    chname = chs(j).ChannelName;
                    TF = isKey(dataSeg,chname);
                    if TF
                        tmp = chNameDataDic(chname);
                        tmp(:,pOffset+1:pOffset+durNew) = dataSeg(chname);
                        chNameDataDic(chname) = tmp;
                    end
                end
            end
            obj.folderNSFDataDic = chNameDataDic;
            data = chNameDataDic;
        end

        function data = getFolderDataTimeRangePartialChannal(obj,startSec,durationSec,chNamesAr)
			hd = obj.folderNSFInfo;
            nsfInfo = hd.nsfInfo;
            
            %prepare the data struct in NSF folder Level  
            chs = nsfInfo.channels;  
            chCountAll = nsfInfo.chCount;
            durAll = nsfInfo.DurationCal;
          
			startReadSec = startSec;
			durationRead = durationSec;
			if(durationSec>durAll)
				durationRead = durAll;
			end
			%initial
			chCntRead = length(chNamesAr);
			stopReadSec = startReadSec + durationRead;
			chDic = nsfInfo.chInfoDic;
			chNameDataDic = containers.Map();
            for i=1:chCntRead
				chname = chNamesAr(i);
				chInfo = chDic(char(chname));
                rows = chInfo.SampleRate;
                cols = durationRead;
                chNameDataDic(char(chname)) = zeros(rows,cols);
            end
            %merge
            segCnt = obj.folderNSFInfo.segCount;
            for i = 1:segCnt
                seg = nsfInfo.segs(i);
                fName = seg.SegmentFolderName;
                start = seg.StartSecond;
                dur = seg.Duration;
				stop = start + dur;
				if start >= stopReadSec
					break;
				end

				if stop <=startReadSec
                    continue;
					%break;
				end

				startNew = start;
				if start < startReadSec
					startNew = startReadSec;
				end

				stopNew = stop;
				if stop > stopReadSec
					stopNew = stopReadSec;
				end
				durNew = stopNew - startNew;
                startN = startNew - seg.StartSecond;
                ndf = fileNDF(fullfile(obj.folderPath, fName));
                dataSeg = ndf.readPeriodChannelRangePhysicalValues(startN,durNew,chNamesAr);
                pOffset = startNew-startSec;
                for j = 1:chCntRead
                    chname = char(chNamesAr(j));
                    TF = isKey(dataSeg,chname);
                    if TF
                        tmp = chNameDataDic(chname);
                        tmp(:,pOffset+1:pOffset+durNew) = dataSeg(chname);
                        chNameDataDic(chname) = tmp;
                    end
                end
            end
            obj.folderNSFDataDic = chNameDataDic;
            data = chNameDataDic;
        end
    end
end

