classdef NDFDecompression<handle
    properties
        huffmanDicID
        huffmanDic
        symbols
    end
    methods
        function obj = NDFDecompression()
            obj.huffmanDicID = uint64(0);
            obj.huffmanDic = cell(256,2);
            obj.symbols = uint8((0:255));
        end

       function hDic = GenHuffmanTree(obj,hFreID,hFreq)
           if hFreID == obj.huffmanDicID
                hDic = obj.huffmanDic;
                return;
           else
                S = sum(hFreq,"all");
                prob = hFreq/S;
                %[dict,avglen] = huffmandict(obj.symbols,prob');
                dictStruct = huffmanDict(obj.symbols,prob');
                hCodes = dictStruct.output;
                
                for i = 1:length(hCodes)
                    codeI = hCodes(i);
                    charI =char(num2cell(codeI{1}));
                    codeAr =reshape(str2num(charI),1,[]);
                    obj.huffmanDic{i,1} = obj.symbols(i);
                    obj.huffmanDic{i,2}=codeAr;
                end
                hDic = obj.huffmanDic;
            end
       end       
       function decRes = DecompressHVZD(obj,dataIn,hDic,rows,cols)
           dataInBi = de2bi(dataIn,[],'left-msb');
           codeAr =reshape(dataInBi',1,[]);
           dataHDec = huffmandeco(double(codeAr),hDic);
           [vDRes,lenVD] = obj.VarintDecode(dataHDec);
           [zDRes,lenZD] = obj.ZigZagDecode(vDRes);
           decRes = obj.cumsum(zDRes(1:rows*cols),rows,cols);
       end

       function decRes = DecompressVZD(obj,dataIn,rows,cols)
           vDRes = zeros(rows*cols);
           lenVD = obj.VarintDecode(dataIn,vDRes);
           zDRes = zeros(rows*cols);
           lenZD = obj.ZigZagDecode(vDRes(1:lenVD),zDRes);
           decRes = obj.cumsum(zDRes,rows,cols);
       end
       
       function [dataDec,lenDec] = VarintDecode(obj,dataIn)
           dLen = length(dataIn);
           offset = 0;
           res = 0;
           lenDec = 1;
           lab = int32(0x80);
           lab2 = int32(0x7F);
           dataInInt = int32(dataIn);
           for i = 1:dLen
               iData = dataInInt(i);
               if bitand(iData, lab) == lab
                   res = bitor(res, bitshift(bitand(iData,lab2),offset));
                   offset = offset+7;
               else
                   res = bitor(res,bitshift(iData,offset));
                   dataDec(lenDec) = res;
                   lenDec=lenDec+1;
                   res = 0;
                   offset = 0;
               end
           end        
       end
       
       function [dataDec,lenDec] = ZigZagDecode(obj,dataIn)
           dLen = length(dataIn);
           lenDec = 1;
           lab = int32(0x01);
           for i=1:dLen
               v = dataIn(i);
               res = bitxor(bitshift(v,-1),(-1*bitand(v,lab)));
               dataDec(lenDec) = res;
               lenDec = lenDec+1;
           end  
       end
       
       function res = cumsum(obj,dataIn,rows,cols)
           B = reshape(dataIn,[cols,rows]);
           B2 = B';
           for i = 2:cols
              B2(:,i) = B2(:,i)+B2(:,i-1); 
           end
           res = B2;
       end
   end
end
